@@ -1,7 +1,7 @@
 /*
  * tomods.h
  *
- * Copyright (c) Chris Putnam 2012-2017
+ * Copyright (c) Chris Putnam 2012-2021
  *
  * Program and source code released under the GPL version 2
  *
@@ -9,7 +9,7 @@
 #ifndef TOMODS_H
 #define TOMODS_H
 
-extern void tomods_processargs( int *argc, char *argv[], param *p,
+void tomods_processargs( int *argc, char *argv[], param *p,
         char *help1, char *help2 );
 
 
