@@ -1,7 +1,7 @@
 /*
  * endxmlin.c
  *
- * Copyright (c) Chris Putnam 2006-2017
+ * Copyright (c) Chris Putnam 2006-2021
  *
  * Program and source code released under the GPL version 2
  *
@@ -26,8 +26,8 @@ extern variants end_all[];
 extern int end_nall;
 
 static int endxmlin_readf( FILE *fp, char *buf, int bufsize, int *bufpos, str *line, str *reference, int *fcharset );
-static int endxmlin_processf( fields *endin, char *p, char *filename, long nref, param *pm );
-extern int endin_typef( fields *endin, char *filename, int nrefs, param *p );
+static int endxmlin_processf( fields *endin, const char *p, const char *filename, long nref, param *pm );
+extern int endin_typef( fields *endin, const char *filename, int nrefs, param *p );
 extern int endin_convertf( fields *endin, fields *info, int reftype, param *p );
 extern int endin_cleanf( bibl *bin, param *p );
 
@@ -35,33 +35,38 @@ extern int endin_cleanf( bibl *bin, param *p );
 /*****************************************************
  PUBLIC: void endxmlin_initparams()
 *****************************************************/
-void
-endxmlin_initparams( param *p, const char *progname )
+int
+endxmlin_initparams( param *pm, const char *progname )
 {
-	p->readformat       = BIBL_ENDNOTEXMLIN;
-	p->charsetin        = BIBL_CHARSET_DEFAULT;
-	p->charsetin_src    = BIBL_SRC_DEFAULT;
-	p->latexin          = 0;
-	p->xmlin            = 1;
-	p->utf8in           = 1;
-	p->nosplittitle     = 0;
-	p->verbose          = 0;
-	p->addcount         = 0;
-	p->output_raw       = 0;
+	pm->readformat       = BIBL_ENDNOTEXMLIN;
+	pm->charsetin        = BIBL_CHARSET_DEFAULT;
+	pm->charsetin_src    = BIBL_SRC_DEFAULT;
+	pm->latexin          = 0;
+	pm->xmlin            = 1;
+	pm->utf8in           = 1;
+	pm->nosplittitle     = 0;
+	pm->verbose          = 0;
+	pm->addcount         = 0;
+	pm->output_raw       = 0;
 
-	p->readf    = endxmlin_readf;
-	p->processf = endxmlin_processf;
-	p->cleanf   = NULL;
-	p->typef    = endin_typef;
-	p->convertf = endin_convertf;
-	p->all      = end_all;
-	p->nall     = end_nall;
+	pm->readf    = endxmlin_readf;
+	pm->processf = endxmlin_processf;
+	pm->cleanf   = NULL;
+	pm->typef    = endin_typef;
+	pm->convertf = endin_convertf;
+	pm->all      = end_all;
+	pm->nall     = end_nall;
 
-	slist_init( &(p->asis) );
-	slist_init( &(p->corps) );
+	slist_init( &(pm->asis) );
+	slist_init( &(pm->corps) );
 
-	if ( !progname ) p->progname = NULL;
-	else p->progname = strdup( progname );
+	if ( !progname ) pm->progname = NULL;
+	else {
+		pm->progname = strdup( progname );
+		if ( !pm->progname ) return BIBL_ERR_MEMERR;
+	}
+
+	return BIBL_OK;
 }
 
 /*****************************************************
@@ -78,17 +83,25 @@ xml_readmore( FILE *fp, char *buf, int bufsize, int *bufpos )
 static int
 endxmlin_readf( FILE *fp, char *buf, int bufsize, int *bufpos, str *line, str *reference, int *fcharset )
 {
-	str tmp;
-	char *startptr = NULL, *endptr = NULL;
 	int haveref = 0, inref = 0, done = 0, file_charset = CHARSET_UNKNOWN, m;
+	char *startptr = NULL, *endptr = NULL;
+	str tmp;
+
 	str_init( &tmp );
+
 	while ( !haveref && !done ) {
-		if ( line->data ) {
-			if ( !inref ) {
-				startptr = xml_findstart( line->data, "RECORD" );
-				if ( startptr ) inref = 1;
-			} else
-				endptr = xml_findend( line->data, "RECORD" );
+
+		if ( str_is_empty( line ) ) {
+			done = xml_readmore( fp, buf, bufsize, bufpos );
+			str_strcatc( line, buf );
+		}
+
+		if ( !inref ) {
+			startptr = xml_find_start( str_cstr( line ), "RECORD" );
+			if ( startptr ) inref = 1;
+		}
+		else {
+			endptr = xml_find_end( str_cstr( line ), "RECORD" );
 		}
 
 		/* If no <record> tag, we can trim up to last 8 bytes */
@@ -102,26 +115,30 @@ endxmlin_readf( FILE *fp, char *buf, int bufsize, int *bufpos, str *line, str *r
 			}
 		}
 
+		/* ...entire reference is not in line, read more */
 		if ( !startptr || !endptr ) {
 			done = xml_readmore( fp, buf, bufsize, bufpos );
 			str_strcatc( line, buf );
-		} else {
-			/* we can reallocate in the str_strcat, so re-find */
-			startptr = xml_findstart( line->data, "RECORD" );
-			endptr = xml_findend( line->data, "RECORD" );
+		}
+		/* ...we can reallocate in str_strcat; must re-find the tags */
+		else {
+			startptr = xml_find_start( str_cstr( line ), "RECORD" );
+			endptr   = xml_find_end  ( str_cstr( line ), "RECORD" );
 			str_segcpy( reference, startptr, endptr );
 			/* clear out information in line */
 			str_strcpyc( &tmp, endptr );
 			str_strcpy( line, &tmp );
 			haveref = 1;
 		}
-		if ( line->data ) {
-			m = xml_getencoding( line );
-			if ( m!=CHARSET_UNKNOWN ) file_charset = m;
-		}
+
+		m = xml_getencoding( line );
+		if ( m!=CHARSET_UNKNOWN ) file_charset = m;
+
 	}
+
 	str_free( &tmp );
 	*fcharset = file_charset;
+
 	return haveref;
 }
 
@@ -151,15 +168,15 @@ endxmlin_datar( xml *node, str *s )
 {
 	int status;
 
-	if ( str_has_value( node->value ) ) {
-		str_strcat( s, node->value );
+	if ( xml_has_value( node ) ) {
+		str_strcat( s, &(node->value) );
 		if ( str_memerr( s ) ) return BIBL_ERR_MEMERR;
 	}
-	if ( node->down && xml_tagexact( node->down, "style" ) ) {
+	if ( node->down && xml_tag_matches( node->down, "style" ) ) {
 		status = endxmlin_datar( node->down, s );
 		if ( status!=BIBL_OK ) return status;
 	}
-	if ( xml_tagexact( node, "style" ) && node->next ) {
+	if ( xml_tag_matches( node, "style" ) && node->next ) {
 		status = endxmlin_datar( node->next, s );
 		if ( status!=BIBL_OK ) return status;
 	}
@@ -179,7 +196,7 @@ endxmlin_data( xml *node, char *inttag, fields *info, int level )
 	if ( status!=BIBL_OK ) return status;
 
 	if ( str_has_value( &s ) ) {
-		status = fields_add( info, inttag, s.data, level );
+		status = fields_add( info, inttag, str_cstr( &s ), level );
 		if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
 	}
 
@@ -192,37 +209,52 @@ endxmlin_data( xml *node, char *inttag, fields *info, int level )
  *       <style>ACTUAL TITLE HERE</style><style>MORE TITLE</style>
  *    </title>
  * </titles>
+ *
+ * or
+ *
+ * <titles>
+ *    <title>ACTUAL TITLE HERE</title>
+ * </titles>
  */
 static int
 endxmlin_titles( xml *node, fields *info )
 {
 	attribs a[] = {
-		{ "title", "%T" },
-		{ "secondary-title", "%B" },
-		{ "tertiary-title", "%S" },
-		{ "alt-title", "%!" },
-		{ "short-title", "SHORTTITLE" },
+		{ "title",           "%T"         },
+		{ "secondary-title", "%B"         },
+		{ "tertiary-title",  "%S"         },
+		{ "alt-title",       "%!"         },
+		{ "short-title",     "SHORTTITLE" },
 	};
-	int i, status, n = sizeof( a ) / sizeof ( a[0] );
+	int n = sizeof( a ) / sizeof ( a[0] );
+	int i, fstatus, status = BIBL_OK;
 	str title;
+
 	str_init( &title );
+
 	for ( i=0; i<n; ++i ) {
-		if ( xml_tagexact( node, a[i].attrib ) && node->down ) {
+		if ( xml_tag_matches( node, a[i].attrib ) ) {
 			str_empty( &title );
-			status = endxmlin_datar( node, &title );
-			if ( status!=BIBL_OK ) return BIBL_ERR_MEMERR;
+			fstatus = endxmlin_datar( node, &title );
+			if ( fstatus!=BIBL_OK ) {
+				status = BIBL_ERR_MEMERR;
+				goto out;
+			}
 			str_trimstartingws( &title );
 			str_trimendingws( &title );
-			status = fields_add( info, a[i].internal, title.data, 0);
-			if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
+			fstatus = fields_add( info, a[i].internal, str_cstr( &title ), LEVEL_MAIN );
+			if ( fstatus!=FIELDS_OK ) {
+				status = BIBL_ERR_MEMERR;
+				goto out;
+			}
 		}
 	}
-	if ( node->next ) {
-		status = endxmlin_titles( node->next, info );
-		if ( status!=BIBL_OK ) return status;
-	}
+
+	if ( node->next ) status = endxmlin_titles( node->next, info );
+
+out:
 	str_free( &title );
-	return BIBL_OK;
+	return status;
 }
 
 /* <contributors>
@@ -247,13 +279,22 @@ endxmlin_titles( xml *node, fields *info )
  *
  */
 static int
-endxmlin_contributor( xml *node, fields *info, char *int_tag, int level )
+endxmlin_contributor( xml *node, fields *info, char *internal_tag, int level )
 {
 	int status;
-	status = endxmlin_data( node, int_tag, info, level );
+
+	/* some values have ^M end-of-line characters in them--replace newline/carriage return with spaces */
+	str_findreplace( xml_value( node ), "\r\n", " " );
+	str_findreplace( xml_value( node ), "\r", " " );
+	str_findreplace( xml_value( node ), "\n", " " );
+
+	/* separate multiple names (separated by "and") with "|" for later parsing by the routines in name.c */
+	str_findreplace( xml_value( node ), " and ", "|" );
+
+	status = endxmlin_data( node, internal_tag, info, level );
 	if ( status!=BIBL_OK ) return status;
 	if ( node->next ) {
-		status = endxmlin_contributor( node->next, info, int_tag, level );
+		status = endxmlin_contributor( node->next, info, internal_tag, level );
 		if ( status!=BIBL_OK ) return status;
 	}
 	return BIBL_OK;
@@ -263,15 +304,15 @@ static int
 endxmlin_contributors( xml *node, fields *info )
 {
 	attribs a[] = {
-		{ "authors", "%A" },
-		{ "secondary-authors", "%E" },
-		{ "tertiary-authors", "%Y" },
+		{ "authors",            "%A" },
+		{ "secondary-authors",  "%E" },
+		{ "tertiary-authors",   "%Y" },
 		{ "subsidiary-authors", "%?" },
 		{ "translated-authors", "%?" },
 	};
 	int i, status, n = sizeof( a ) / sizeof ( a[0] );
 	for ( i=0; i<n; ++i ) {
-		if ( xml_tagexact( node, a[i].attrib ) && node->down ) {
+		if ( xml_tag_matches( node, a[i].attrib ) && node->down ) {
 			status = endxmlin_contributor( node->down, info, a[i].internal, 0 );
 			if ( status!=BIBL_OK ) return status;
 		}
@@ -287,8 +328,8 @@ static int
 endxmlin_keyword( xml *node, fields *info )
 {
 	int status;
-	if ( xml_tagexact( node, "keyword" ) ) {
-		status = endxmlin_data( node, "%K", info, 0 );
+	if ( xml_tag_matches( node, "keyword" ) ) {
+		status = endxmlin_data( node, "%K", info, LEVEL_MAIN );
 		if ( status!=BIBL_OK ) return status;
 	}
 	if ( node->next ) {
@@ -301,7 +342,7 @@ endxmlin_keyword( xml *node, fields *info )
 static int
 endxmlin_keywords( xml *node, fields *info )
 {
-	if ( node->down && xml_tagexact( node->down, "keyword" ) )
+	if ( node->down && xml_tag_matches( node->down, "keyword" ) )
 		return endxmlin_keyword( node->down, info );
 	return BIBL_OK;
 }
@@ -313,16 +354,16 @@ endxmlin_keywords( xml *node, fields *info )
 static int
 endxmlin_ern( xml *node, fields *info )
 {
-	if ( xml_tagexact( node, "electronic-resource-num" ) )
-		return endxmlin_data( node, "DOI", info, 0 );
+	if ( xml_tag_matches( node, "electronic-resource-num" ) )
+		return endxmlin_data( node, "DOI", info, LEVEL_MAIN );
 	return BIBL_OK;
 }
 
 static int
 endxmlin_language( xml *node, fields *info )
 {
-	if ( xml_tagexact( node, "language" ) )
-		return endxmlin_data( node, "%G", info, 0 );
+	if ( xml_tag_matches( node, "language" ) )
+		return endxmlin_data( node, "%G", info, LEVEL_MAIN );
 	return BIBL_OK;
 }
 
@@ -337,8 +378,8 @@ static int
 endxmlin_fileattach( xml *node, fields *info )
 {
 	int status;
-	if ( xml_tagexact( node, "url" ) ) {
-		status = endxmlin_data( node, "FILEATTACH", info, 0 );
+	if ( xml_tag_matches( node, "url" ) ) {
+		status = endxmlin_data( node, "FILEATTACH", info, LEVEL_MAIN );
 		if ( status!=BIBL_OK ) return status;
 	}
 	if ( node->down ) {
@@ -356,17 +397,17 @@ static int
 endxmlin_urls( xml *node, fields *info )
 {
 	int status;
-	if ( xml_tagexact( node, "pdf-urls" ) && node->down ) {
+	if ( xml_tag_matches( node, "pdf-urls" ) && node->down ) {
 		status = endxmlin_fileattach( node->down, info );
 		if ( status!=BIBL_OK ) return status;
-	} else if ( xml_tagexact( node, "url" ) ) {
-		status = endxmlin_data( node, "%U", info, 0 );
+	} else if ( xml_tag_matches( node, "url" ) ) {
+		status = endxmlin_data( node, "%U", info, LEVEL_MAIN );
 		if ( status!=BIBL_OK ) return status;
 	} else {
 		if ( node->down ) {
-			if ( xml_tagexact( node->down, "related-urls" ) ||
-			     xml_tagexact( node->down, "pdf-urls" ) ||
-			     xml_tagexact( node->down, "url" ) ) {
+			if ( xml_tag_matches( node->down, "related-urls" ) ||
+			     xml_tag_matches( node->down, "pdf-urls" ) ||
+			     xml_tag_matches( node->down, "url" ) ) {
 				status = endxmlin_urls( node->down, info );
 				if ( status!=BIBL_OK ) return status;
 			}
@@ -382,10 +423,10 @@ endxmlin_urls( xml *node, fields *info )
 static int
 endxmlin_pubdates( xml *node, fields *info )
 {
-	if ( xml_tagexact( node, "date" ) )
-		return endxmlin_data( node, "%8", info, 0 );
+	if ( xml_tag_matches( node, "date" ) )
+		return endxmlin_data( node, "%8", info, LEVEL_MAIN );
 	else {
-		if ( node->down && xml_tagexact( node->down, "date" ) )
+		if ( node->down && xml_tag_matches( node->down, "date" ) )
 			return endxmlin_pubdates( node->down, info );
 	}
 	return BIBL_OK;
@@ -395,18 +436,18 @@ static int
 endxmlin_dates( xml *node, fields *info )
 {
 	int status;
-	if ( xml_tagexact( node, "year" ) ) {
-		status = endxmlin_data( node, "%D", info, 0 );
+	if ( xml_tag_matches( node, "year" ) ) {
+		status = endxmlin_data( node, "%D", info, LEVEL_MAIN );
 		if ( status!=BIBL_OK ) return status;
 	} else {
 		if ( node->down ) {
-			if ( xml_tagexact( node->down, "year" ) ) {
+			if ( xml_tag_matches( node->down, "year" ) ) {
 				status = endxmlin_dates( node->down, info );
 				if ( status!=BIBL_OK ) return status;
 			}
-			if ( xml_tagexact( node->down, "pub-dates" ) ) {
+			if ( xml_tag_matches( node->down, "pub-dates" ) ) {
 				status = endxmlin_pubdates( node->down, info );
-				if ( status!=BIBL_OK );
+				if ( status!=BIBL_OK ) return status;
 			}
 		}
 	}
@@ -426,9 +467,9 @@ endxmlin_reftype( xml *node, fields *info )
 	int status;
 	str *s;
 
-	s = xml_getattrib( node, "name" );
+	s = xml_attribute( node, "name" );
 	if ( str_has_value( s ) ) {
-		status = fields_add( info, "%0", str_cstr( s ), 0 );
+		status = fields_add( info, "%0", str_cstr( s ), LEVEL_MAIN );
 		if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
 	}
 
@@ -439,89 +480,90 @@ static int
 endxmlin_record( xml *node, fields *info )
 {
 	attribs a[] = {
-		{ "volume", "%V" },
-		{ "num-vol", "%6" },
-		{ "pages",  "%P" },
-		{ "number", "%N" },
-		{ "issue",  "%N" },
-		{ "label",  "%F" },
-		{ "auth-address", "%C" },
+		{ "volume",           "%V" },
+		{ "num-vol",          "%6" },
+		{ "pages",            "%P" },
+		{ "number",           "%N" },
+		{ "issue",            "%N" },
+		{ "label",            "%F" },
+		{ "auth-address",     "%C" },
 		{ "auth-affiliation", "%C" },
-		{ "pub-location", "%C" },
-		{ "publisher", "%I" },
-		{ "abstract", "%X" },
-		{ "edition", "%7" },
-		{ "reprint-edition", "%)" },
-		{ "section", "%&" },
-		{ "accession-num", "%M" },
-		{ "call-num", "%L" },
-		{ "isbn", "%@" },
-		{ "notes", "%O" },
-		{ "custom1", "%1" },
-		{ "custom2", "%2" },
-		{ "custom3", "%3" },
-		{ "custom4", "%4" },
-		{ "custom5", "%#" },
-		{ "custom6", "%$" },
+		{ "pub-location",     "%C" },
+		{ "publisher",        "%I" },
+		{ "abstract",         "%X" },
+		{ "edition",          "%7" },
+		{ "reprint-edition",  "%)" },
+		{ "section",          "%&" },
+		{ "accession-num",    "%M" },
+		{ "call-num",         "%L" },
+		{ "isbn",             "%@" },
+		{ "notes",            "%O" },
+		{ "custom1",          "%1" },
+		{ "custom2",          "%2" },
+		{ "custom3",          "%3" },
+		{ "custom4",          "%4" },
+		{ "custom5",          "%#" },
+		{ "custom6",          "%$" },
 	};
 	int i, status, n = sizeof ( a ) / sizeof( a[0] );
-	if ( xml_tagexact( node, "DATABASE" ) ) {
-	} else if ( xml_tagexact( node, "SOURCE-APP" ) ) {
-	} else if ( xml_tagexact( node, "REC-NUMBER" ) ) {
-	} else if ( xml_tagexact( node, "ref-type" ) ) {
+
+	if ( xml_tag_matches( node, "DATABASE" ) ) {
+	} else if ( xml_tag_matches( node, "SOURCE-APP" ) ) {
+	} else if ( xml_tag_matches( node, "REC-NUMBER" ) ) {
+	} else if ( xml_tag_matches( node, "ref-type" ) ) {
 		status = endxmlin_reftype( node, info );
 		if ( status!=BIBL_OK ) return status;
-	} else if ( xml_tagexact( node, "contributors" ) ) {
+	} else if ( xml_tag_matches( node, "contributors" ) ) {
 		if ( node->down ) {
 			status = endxmlin_contributors( node->down, info );
 			if ( status!=BIBL_OK ) return status;
 		}
-	} else if ( xml_tagexact( node, "titles" ) ) {
+	} else if ( xml_tag_matches( node, "titles" ) ) {
 		if ( node->down ) endxmlin_titles( node->down, info );
-	} else if ( xml_tagexact( node, "keywords" ) ) {
+	} else if ( xml_tag_matches( node, "keywords" ) ) {
 		status = endxmlin_keywords( node, info );
 		if ( status!=BIBL_OK ) return status;
-	} else if ( xml_tagexact( node, "urls" ) ) {
+	} else if ( xml_tag_matches( node, "urls" ) ) {
 		status = endxmlin_urls( node, info );
 		if ( status!=BIBL_OK ) return status;
-	} else if ( xml_tagexact( node, "electronic-resource-num" ) ) {
+	} else if ( xml_tag_matches( node, "electronic-resource-num" ) ) {
 		status = endxmlin_ern( node, info );
 		if ( status!=BIBL_OK ) return status;
-	} else if ( xml_tagexact( node, "dates" ) ) {
+	} else if ( xml_tag_matches( node, "dates" ) ) {
 		status = endxmlin_dates( node, info );
 		if ( status!=BIBL_OK ) return status;
-	} else if ( xml_tagexact( node, "language" ) ) {
+	} else if ( xml_tag_matches( node, "language" ) ) {
 		status = endxmlin_language( node, info );
 		if ( status!=BIBL_OK ) return status;
-	} else if ( xml_tagexact( node, "periodical" ) ) {
-	} else if ( xml_tagexact( node, "secondary-volume" ) ) {
-	} else if ( xml_tagexact( node, "secondary-issue" ) ) {
-	} else if ( xml_tagexact( node, "reprint-status" ) ) {
-	} else if ( xml_tagexact( node, "orig-pub" ) ) {
-	} else if ( xml_tagexact( node, "report-id" ) ) {
-	} else if ( xml_tagexact( node, "coden" ) ) {
-	} else if ( xml_tagexact( node, "caption" ) ) {
-	} else if ( xml_tagexact( node, "research-notes" ) ) {
-	} else if ( xml_tagexact( node, "work-type" ) ) {
-	} else if ( xml_tagexact( node, "reviewed-item" ) ) {
-	} else if ( xml_tagexact( node, "availability" ) ) {
-	} else if ( xml_tagexact( node, "remote-source" ) ) {
-	} else if ( xml_tagexact( node, "meeting-place" ) ) {
-	} else if ( xml_tagexact( node, "work-location" ) ) {
-	} else if ( xml_tagexact( node, "work-extent" ) ) {
-	} else if ( xml_tagexact( node, "pack-method" ) ) {
-	} else if ( xml_tagexact( node, "size" ) ) {
-	} else if ( xml_tagexact( node, "repro-ratio" ) ) {
-	} else if ( xml_tagexact( node, "remote-database-name" ) ) {
-	} else if ( xml_tagexact( node, "remote-database-provider" ) ) {
-	} else if ( xml_tagexact( node, "access-date" ) ) {
-	} else if ( xml_tagexact( node, "modified-data" ) ) {
-	} else if ( xml_tagexact( node, "misc1" ) ) {
-	} else if ( xml_tagexact( node, "misc2" ) ) {
-	} else if ( xml_tagexact( node, "misc3" ) ) {
+	} else if ( xml_tag_matches( node, "periodical" ) ) {
+	} else if ( xml_tag_matches( node, "secondary-volume" ) ) {
+	} else if ( xml_tag_matches( node, "secondary-issue" ) ) {
+	} else if ( xml_tag_matches( node, "reprint-status" ) ) {
+	} else if ( xml_tag_matches( node, "orig-pub" ) ) {
+	} else if ( xml_tag_matches( node, "report-id" ) ) {
+	} else if ( xml_tag_matches( node, "coden" ) ) {
+	} else if ( xml_tag_matches( node, "caption" ) ) {
+	} else if ( xml_tag_matches( node, "research-notes" ) ) {
+	} else if ( xml_tag_matches( node, "work-type" ) ) {
+	} else if ( xml_tag_matches( node, "reviewed-item" ) ) {
+	} else if ( xml_tag_matches( node, "availability" ) ) {
+	} else if ( xml_tag_matches( node, "remote-source" ) ) {
+	} else if ( xml_tag_matches( node, "meeting-place" ) ) {
+	} else if ( xml_tag_matches( node, "work-location" ) ) {
+	} else if ( xml_tag_matches( node, "work-extent" ) ) {
+	} else if ( xml_tag_matches( node, "pack-method" ) ) {
+	} else if ( xml_tag_matches( node, "size" ) ) {
+	} else if ( xml_tag_matches( node, "repro-ratio" ) ) {
+	} else if ( xml_tag_matches( node, "remote-database-name" ) ) {
+	} else if ( xml_tag_matches( node, "remote-database-provider" ) ) {
+	} else if ( xml_tag_matches( node, "access-date" ) ) {
+	} else if ( xml_tag_matches( node, "modified-data" ) ) {
+	} else if ( xml_tag_matches( node, "misc1" ) ) {
+	} else if ( xml_tag_matches( node, "misc2" ) ) {
+	} else if ( xml_tag_matches( node, "misc3" ) ) {
 	} else {
 		for ( i=0; i<n; ++i ) {
-			if ( xml_tagexact( node, a[i].attrib ) ) {
+			if ( xml_tag_matches( node, a[i].attrib ) ) {
 				status = endxmlin_data( node, a[i].internal, info, 0 );
 				if ( status!=BIBL_OK ) return status;
 			}
@@ -538,10 +580,10 @@ static int
 endxmlin_assembleref( xml *node, fields *info )
 {
 	int status;
-	if ( str_is_empty( node->tag ) ) {
+	if ( str_is_empty( &(node->tag) ) ) {
 		if ( node->down )
 			return endxmlin_assembleref( node->down, info );
-	} else if ( xml_tagexact( node, "RECORD" ) ) {
+	} else if ( xml_tag_matches( node, "RECORD" ) ) {
 		if ( node->down ) {
 			status = endxmlin_record( node->down, info );
 			if ( status!=BIBL_OK ) return status;
@@ -557,13 +599,13 @@ endxmlin_assembleref( xml *node, fields *info )
  * as the tags used in the Refer format output
  */
 static int
-endxmlin_processf( fields *fin, char *data, char *filename, long nref, param *pm )
+endxmlin_processf( fields *fin, const char *data, const char *filename, long nref, param *pm )
 {
 	int status;
 	xml top;
 
 	xml_init( &top );
-	xml_tree( data, &top );
+	xml_parse( data, &top );
 	status = endxmlin_assembleref( &top, fin );
 	xml_free( &top );
 
