@@ -1,6 +1,6 @@
 /* generic.h
  *
- * Copyright (c) Chris Putnam 2016-2017
+ * Copyright (c) Chris Putnam 2016-2021
  *
  * Source code released under GPL version 2
  *
@@ -8,6 +8,7 @@
 #ifndef GENERIC_H
 #define GENERIC_H
 
+#include <stdio.h>
 #include "bibutils.h"
 
 int generic_null    ( fields *bibin, int n, str *intag, str *invalue, int level, param *pm, char *outtag, fields *bibout );
@@ -19,5 +20,8 @@ int generic_serialno( fields *bibin, int n, str *intag, str *invalue, int level,
 int generic_simple  ( fields *bibin, int n, str *intag, str *invalue, int level, param *pm, char *outtag, fields *bibout );
 int generic_skip    ( fields *bibin, int n, str *intag, str *invalue, int level, param *pm, char *outtag, fields *bibout );
 int generic_title   ( fields *bibin, int n, str *intag, str *invalue, int level, param *pm, char *outtag, fields *bibout );
+int generic_genre   ( fields *bibin, int n, str *intag, str *invalue, int level, param *pm, char *outtag, fields *bibout );
+
+void generic_writeheader( FILE *outptr, param *pm );
 
 #endif
