@@ -1,7 +1,7 @@
 /*
  * risout.c
  *
- * Copyright (c) Chris Putnam 2003-2017
+ * Copyright (c) Chris Putnam 2003-2021
  *
  * Source code released under the GPL version 2
  *
@@ -9,45 +9,63 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
-#include "utf8.h"
-#include "str.h"
-#include "strsearch.h"
+#include "append_easy.h"
+#include "bibformats.h"
 #include "fields.h"
+#include "generic.h"
 #include "name.h"
+#include "str.h"
 #include "title.h"
 #include "url.h"
-#include "bibformats.h"
+#include "utf8.h"
+
+/*****************************************************
+ PUBLIC: int risout_initparams()
+*****************************************************/
 
 static int  risout_write( fields *info, FILE *fp, param *p, unsigned long refnum );
-static void risout_writeheader( FILE *outptr, param *p );
+static int  risout_assemble( fields *in, fields *out, param *pm, unsigned long refnum );
 
-
-void
-risout_initparams( param *p, const char *progname )
+int
+risout_initparams( param *pm, const char *progname )
 {
-	p->writeformat      = BIBL_RISOUT;
-	p->format_opts      = 0;
-	p->charsetout       = BIBL_CHARSET_DEFAULT;
-	p->charsetout_src   = BIBL_SRC_DEFAULT;
-	p->latexout         = 0;
-	p->utf8out          = BIBL_CHARSET_UTF8_DEFAULT;
-	p->utf8bom          = BIBL_CHARSET_BOM_DEFAULT;
-	p->xmlout           = BIBL_XMLOUT_FALSE;
-	p->nosplittitle     = 0;
-	p->verbose          = 0;
-	p->addcount         = 0;
-	p->singlerefperfile = 0;
+	pm->writeformat      = BIBL_RISOUT;
+	pm->format_opts      = 0;
+	pm->charsetout       = BIBL_CHARSET_DEFAULT;
+	pm->charsetout_src   = BIBL_SRC_DEFAULT;
+	pm->latexout         = 0;
+	pm->utf8out          = BIBL_CHARSET_UTF8_DEFAULT;
+	pm->utf8bom          = BIBL_CHARSET_BOM_DEFAULT;
+	pm->xmlout           = BIBL_XMLOUT_FALSE;
+	pm->nosplittitle     = 0;
+	pm->verbose          = 0;
+	pm->addcount         = 0;
+	pm->singlerefperfile = 0;
 
-	if ( p->charsetout == BIBL_CHARSET_UNICODE ) {
-		p->utf8out = p->utf8bom = 1;
+	if ( pm->charsetout == BIBL_CHARSET_UNICODE ) {
+		pm->utf8out = pm->utf8bom = 1;
 	}
 
-	p->headerf = risout_writeheader;
-	p->footerf = NULL;
-	p->writef  = risout_write;
+	pm->headerf   = generic_writeheader;
+	pm->footerf   = NULL;
+	pm->assemblef = risout_assemble;
+	pm->writef    = risout_write;
+
+	if ( !pm->progname ) {
+		if ( progname==NULL ) pm->progname = NULL;
+		else {
+			pm->progname = strdup( progname );
+			if ( !pm->progname ) return BIBL_ERR_MEMERR;
+		}
+	}
+
+	return BIBL_OK;
 }
 
+/*****************************************************
+ PUBLIC: int risout_assemble()
+*****************************************************/
+
 enum { 
 	TYPE_UNKNOWN = 0,
 	TYPE_STD,                /* standard/generic */
@@ -69,6 +87,7 @@ enum {
 	TYPE_REPORT,             /* report */
 	TYPE_STATUTE,            /* statute */
 	TYPE_THESIS,             /* thesis */
+	TYPE_LICENTIATETHESIS,   /* thesis */
 	TYPE_MASTERSTHESIS,      /* thesis */
 	TYPE_PHDTHESIS,          /* thesis */
 	TYPE_DIPLOMATHESIS,      /* thesis */
@@ -119,6 +138,7 @@ write_type( FILE *fp, int type )
 		[ TYPE_REPORT             ] = "TYPE_REPORT",
 		[ TYPE_STATUTE            ] = "TYPE_STATUTE",
 		[ TYPE_THESIS             ] = "TYPE_THESIS",
+		[ TYPE_LICENTIATETHESIS   ] = "TYPE_LICENTIATETHESIS",
 		[ TYPE_MASTERSTHESIS      ] = "TYPE_MASTERSTHESIS",
 		[ TYPE_PHDTHESIS          ] = "TYPE_PHDTHESIS",
 		[ TYPE_DIPLOMATHESIS      ] = "TYPE_DIPLOMATHESIS",
@@ -145,7 +165,7 @@ static void
 verbose_type_assignment( char *tag, char *value, param *p, int type )
 {
 	if ( p->progname ) fprintf( stderr, "%s: ", p->progname );
-	fprintf( stderr, "Type from tag '%s' data '%s': ", tag, value );
+	fprintf( stderr, "Type from tag '%s' value '%s': ", tag, value );
 	write_type( stderr, type );
 	fprintf( stderr, "\n" );
 }
@@ -162,28 +182,33 @@ static int
 get_type_genre( fields *f, param *p )
 {
 	match_type match_genres[] = {
-		{ "academic journal",          TYPE_ARTICLE },
-		{ "article",                   TYPE_ARTICLE },
-		{ "journal article",           TYPE_ARTICLE },
-		{ "magazine",                  TYPE_MAGARTICLE },
-		{ "conference publication",    TYPE_CONF },
-		{ "newspaper",                 TYPE_NEWSPAPER },
-		{ "legislation",               TYPE_STATUTE },
-		{ "communication",             TYPE_PCOMM },
-		{ "hearing",                   TYPE_HEAR },
-		{ "electronic",                TYPE_ELEC },
-		{ "legal case and case notes", TYPE_CASE },
-		{ "book chapter",              TYPE_INBOOK },
-		{ "Ph.D. thesis",              TYPE_PHDTHESIS },
-		{ "Masters thesis",            TYPE_MASTERSTHESIS },
-		{ "Diploma thesis",            TYPE_DIPLOMATHESIS },
-		{ "Doctoral thesis",           TYPE_DOCTORALTHESIS },
+		{ "academic journal",          TYPE_ARTICLE            },
+		{ "article",                   TYPE_ARTICLE            },
+		{ "journal article",           TYPE_ARTICLE            },
+		{ "magazine",                  TYPE_MAGARTICLE         },
+		{ "conference publication",    TYPE_CONF               },
+		{ "newspaper",                 TYPE_NEWSPAPER          },
+		{ "legislation",               TYPE_STATUTE            },
+		{ "communication",             TYPE_PCOMM              },
+		{ "hearing",                   TYPE_HEAR               },
+		{ "electronic",                TYPE_ELEC               },
+		{ "legal case and case notes", TYPE_CASE               },
+		{ "book",                      TYPE_BOOK               },
+		{ "collection",                TYPE_BOOK               },
+		{ "book chapter",              TYPE_INBOOK             },
+		{ "Ph.D. thesis",              TYPE_PHDTHESIS          },
+		{ "Licentiate thesis",         TYPE_LICENTIATETHESIS   },
+		{ "Masters thesis",            TYPE_MASTERSTHESIS      },
+		{ "Diploma thesis",            TYPE_DIPLOMATHESIS      },
+		{ "Doctoral thesis",           TYPE_DOCTORALTHESIS     },
 		{ "Habilitation thesis",       TYPE_HABILITATIONTHESIS },
-		{ "report",                    TYPE_REPORT },
-		{ "abstract or summary",       TYPE_ABSTRACT },
-		{ "patent",                    TYPE_PATENT },
-		{ "unpublished",               TYPE_UNPUBLISHED },
-		{ "map",                       TYPE_MAP },
+		{ "report",                    TYPE_REPORT             },
+		{ "technical report",          TYPE_REPORT             },
+		{ "abstract or summary",       TYPE_ABSTRACT           },
+		{ "patent",                    TYPE_PATENT             },
+		{ "unpublished",               TYPE_UNPUBLISHED        },
+		{ "manuscript",                TYPE_UNPUBLISHED        },
+		{ "map",                       TYPE_MAP                },
 	};
 	int nmatch_genres = sizeof( match_genres ) / sizeof( match_genres[0] );
 	char *tag, *value;
@@ -193,25 +218,21 @@ get_type_genre( fields *f, param *p )
 
 	for ( i=0; i<fields_num( f ); ++i ) {
 		tag = ( char * ) fields_tag( f, i, FIELDS_CHRP );
-		if ( strcmp( tag, "GENRE" ) && strcmp( tag, "NGENRE" ) ) continue;
+		if ( strncmp( tag, "GENRE", 5 ) ) continue;
 		value = ( char * ) fields_value( f, i, FIELDS_CHRP );
 		for ( j=0; j<nmatch_genres; ++j )
 			if ( !strcasecmp( match_genres[j].name, value ) )
 				type = match_genres[j].type;
 		if ( p->verbose ) verbose_type_assignment( tag, value, p, type );
-		if ( type==TYPE_UNKNOWN ) {
+		if ( type==TYPE_BOOK ) {
+			if ( fields_level( f, i ) > 0 ) type=TYPE_INBOOK;
+		}
+		/* delay these in lieu of being able to assign something more specific */
+		else if ( type==TYPE_UNKNOWN ) {
 			if ( !strcasecmp( value, "periodical" ) )
 				type = TYPE_ARTICLE;
 			else if ( !strcasecmp( value, "thesis" ) )
 				type = TYPE_THESIS;
-			else if ( !strcasecmp( value, "book" ) ) {
-				if ( fields_level( f, i )==0 ) type=TYPE_BOOK;
-				else type=TYPE_INBOOK;
-			}
-			else if ( !strcasecmp( value, "collection" ) ) {
-				if ( fields_level( f, i )==0 ) type=TYPE_BOOK;
-				else type=TYPE_INBOOK;
-			}
 		}
 
 	}
@@ -264,19 +285,13 @@ get_type_issuance( fields *f, param *p )
 {
 	int type = TYPE_UNKNOWN;
 	int i, monographic = 0, monographic_level = 0;
-//	int text = 0;
 	for ( i=0; i<f->n; ++i ) {
 		if ( !strcasecmp( (char *) fields_tag( f, i, FIELDS_CHRP_NOUSE ), "issuance" ) &&
 		     !strcasecmp( (char *) fields_value( f, i, FIELDS_CHRP_NOUSE ), "MONOGRAPHIC" ) ){
 			monographic = 1;
-			monographic_level = f->level[i];
+			monographic_level = fields_level( f, i );
 		}
-//		if ( !strcasecmp( (char *) fields_tag( f, i, FIELDS_CHRP_NOUSE ), "typeOfResource" ) &&
-//		     !strcasecmp( (char *) fields_value( f, i, FIELDS_CHRP_NOUSE ), "text" ) ) {
-//			text = 1;
-//		}
 	}
-//	if ( monographic && text ) {
 	if ( monographic ) {
 		if ( monographic_level==0 ) type=TYPE_BOOK;
 		else if ( monographic_level>0 ) type=TYPE_INBOOK;
@@ -359,7 +374,7 @@ append_type( int type, param *p, fields *out, int *status )
 }
 
 static void
-append_people( fields *f, char *tag, char *ristag, int level, fields *out, int *status )
+append_people( fields *f, char *tag, int level, fields *out, char *ristag, int *status )
 {
 	vplist_index i;
 	str oneperson;
@@ -459,45 +474,25 @@ static void
 append_pages( fields *in, fields *out, int *status )
 {
 	char *sn, *en, *ar;
-	int fstatus;
 
 	sn = fields_findv( in, LEVEL_ANY, FIELDS_CHRP, "PAGES:START" );
 	en = fields_findv( in, LEVEL_ANY, FIELDS_CHRP, "PAGES:STOP" );
 
 	if ( sn || en ) {
 		if ( sn ) {
-			fstatus = fields_add( out, "SP", sn, LEVEL_MAIN );
-			if ( fstatus!=FIELDS_OK ) *status = BIBL_ERR_MEMERR;
+			*status = append_easypage( out, "SP", sn, LEVEL_MAIN );
 		}
 		if ( en ) {
-			fstatus = fields_add( out, "EP", en, LEVEL_MAIN );
-			if ( fstatus!=FIELDS_OK ) *status = BIBL_ERR_MEMERR;
+			*status = append_easypage( out, "EP", en, LEVEL_MAIN );
 		}
 	} else {
 		ar = fields_findv( in, LEVEL_ANY, FIELDS_CHRP, "ARTICLENUMBER" );
 		if ( ar ) {
-			fstatus = fields_add( out, "SP", ar, LEVEL_MAIN );
-			if ( fstatus!=FIELDS_OK ) *status = BIBL_ERR_MEMERR;
+			*status = append_easypage( out, "SP", ar, LEVEL_MAIN );
 		}
 	}
 }
 
-static void
-append_keywords( fields *in, fields *out, int *status )
-{
-	vplist_index i;
-	int fstatus;
-	vplist vpl;
-
-	vplist_init( &vpl );
-	fields_findv_each( in, LEVEL_ANY, FIELDS_CHRP, &vpl, "KEYWORD" );
-	for ( i=0; i<vpl.n; ++i ) {
-		fstatus = fields_add( out, "KW", ( char * ) vplist_get( &vpl, i ), LEVEL_MAIN );
-		if ( fstatus!=FIELDS_OK ) *status = BIBL_ERR_MEMERR;
-	}
-	vplist_free( &vpl );
-}
-
 static void
 append_urls( fields *in, fields *out, int *status )
 {
@@ -545,6 +540,11 @@ append_thesishint( int type, fields *out, int *status )
 		fstatus = fields_add( out, "U1", "Habilitation thesis", LEVEL_MAIN );
 		if ( fstatus!=FIELDS_OK ) *status = BIBL_ERR_MEMERR;
 	}
+
+	else if ( type==TYPE_LICENTIATETHESIS ) {
+		fstatus = fields_add( out, "U1", "Licentiate thesis", LEVEL_MAIN );
+		if ( fstatus!=FIELDS_OK ) *status = BIBL_ERR_MEMERR;
+	}
 }
 
 static int
@@ -561,7 +561,7 @@ is_uri_scheme( char *p )
 
 
 static void
-append_file( fields *in, char *tag, char *ristag, int level, fields *out, int *status )
+append_file( fields *in, char *tag, int level, fields *out, char *ristag, int *status )
 {
 	vplist_index i;
 	str filename;
@@ -571,6 +571,7 @@ append_file( fields *in, char *tag, char *ristag, int level, fields *out, int *s
 
 	str_init( &filename );
 	vplist_init( &a );
+
 	fields_findv_each( in, level, FIELDS_CHRP, &a, tag );
 	for ( i=0; i<a.n; ++i ) {
 		fl = ( char * ) vplist_get( &a, i );
@@ -581,144 +582,108 @@ append_file( fields *in, char *tag, char *ristag, int level, fields *out, int *s
 		fstatus = fields_add( out, ristag, str_cstr( &filename ), LEVEL_MAIN );
 		if ( fstatus!=FIELDS_OK ) { *status = BIBL_ERR_MEMERR; goto out; }
 	}
+
 out:
 	vplist_free( &a );
 	str_free( &filename );
 }
 
-static void
-append_easy( fields *in, char *tag, char *ristag, int level, fields *out, int *status )
-{
-	char *value;
-	int fstatus;
-
-	value = fields_findv( in, level, FIELDS_CHRP, tag );
-	if ( value ) {
-		fstatus = fields_add( out, ristag, value, LEVEL_MAIN );
-		if ( fstatus!=FIELDS_OK ) *status = BIBL_ERR_MEMERR;
-	}
-}
-
-static void
-append_easyall( fields *in, char *tag, char *ristag, int level, fields *out, int *status )
-{
-	vplist_index i;
-	int fstatus;
-	vplist a;
-
-	vplist_init( &a );
-	fields_findv_each( in, level, FIELDS_CHRP, &a, tag );
-	for ( i=0; i<a.n; ++i ) {
-		fstatus = fields_add( out, ristag, (char *) vplist_get( &a, i ), LEVEL_MAIN );
-		if ( fstatus!=FIELDS_OK ) *status = BIBL_ERR_MEMERR;
-	}
-	vplist_free( &a );
-}
 
 static void
 append_allpeople( fields *in, int type, fields *out, int *status )
 {
-	append_people ( in, "AUTHOR",      "AU", LEVEL_MAIN,   out, status );
-	append_easyall( in, "AUTHOR:CORP", "AU", LEVEL_MAIN,   out, status );
-	append_easyall( in, "AUTHOR:ASIS", "AU", LEVEL_MAIN,   out, status );
-	append_people ( in, "AUTHOR",      "A2", LEVEL_HOST,   out, status );
-	append_easyall( in, "AUTHOR:CORP", "A2", LEVEL_HOST,   out, status );
-	append_easyall( in, "AUTHOR:ASIS", "A2", LEVEL_HOST,   out, status );
-	append_people ( in, "AUTHOR",      "A3", LEVEL_SERIES, out, status );
-	append_easyall( in, "AUTHOR:CORP", "A3", LEVEL_SERIES, out, status );
-	append_easyall( in, "AUTHOR:ASIS", "A3", LEVEL_SERIES, out, status );
-	append_people ( in, "EDITOR",      "ED", LEVEL_MAIN,   out, status );
-	append_easyall( in, "EDITOR:CORP", "ED", LEVEL_MAIN,   out, status );
-	append_easyall( in, "EDITOR:ASIS", "ED", LEVEL_MAIN,   out, status );
+
+	append_people ( in, "AUTHOR",      LEVEL_MAIN,   out, "AU", status );
+	append_easyall( in, "AUTHOR:CORP", LEVEL_MAIN,   out, "AU", status );
+	append_easyall( in, "AUTHOR:ASIS", LEVEL_MAIN,   out, "AU", status );
+
+	append_people ( in, "AUTHOR",      LEVEL_HOST,   out, "A2", status );
+	append_easyall( in, "AUTHOR:CORP", LEVEL_HOST,   out, "A2", status );
+	append_easyall( in, "AUTHOR:ASIS", LEVEL_HOST,   out, "A2", status );
+
+	append_people ( in, "AUTHOR",      LEVEL_SERIES, out, "A3", status );
+	append_easyall( in, "AUTHOR:CORP", LEVEL_SERIES, out, "A3", status );
+	append_easyall( in, "AUTHOR:ASIS", LEVEL_SERIES, out, "A3", status );
+
+	append_people ( in, "EDITOR",      LEVEL_MAIN,   out, "ED", status );
+	append_easyall( in, "EDITOR:CORP", LEVEL_MAIN,   out, "ED", status );
+	append_easyall( in, "EDITOR:ASIS", LEVEL_MAIN,   out, "ED", status );
+
 	if ( type_is_element[ type ] ) {
-		append_people ( in, "EDITOR",      "ED", LEVEL_HOST, out, status );
-		append_easyall( in, "EDITOR:CORP", "ED", LEVEL_HOST, out, status );
-		append_easyall( in, "EDITOR:ASIS", "ED", LEVEL_HOST, out, status );
-	} else {
-		append_people ( in, "EDITOR",      "A3", LEVEL_HOST, out, status );
-		append_easyall( in, "EDITOR:CORP", "A3", LEVEL_HOST, out, status );
-		append_easyall( in, "EDITOR:ASIS", "A3", LEVEL_HOST, out, status );
+		append_people ( in, "EDITOR",      LEVEL_HOST, out, "ED", status );
+		append_easyall( in, "EDITOR:CORP", LEVEL_HOST, out, "ED", status );
+		append_easyall( in, "EDITOR:ASIS", LEVEL_HOST, out, "ED", status );
 	}
-	append_people ( in, "EDITOR",      "A3", LEVEL_SERIES, out, status );
-	append_easyall( in, "EDITOR:CORP", "A3", LEVEL_SERIES, out, status );
-	append_easyall( in, "EDITOR:ASIS", "A3", LEVEL_SERIES, out, status );
+
+	else {
+		append_people ( in, "EDITOR",      LEVEL_HOST, out, "A3", status );
+		append_easyall( in, "EDITOR:CORP", LEVEL_HOST, out, "A3", status );
+		append_easyall( in, "EDITOR:ASIS", LEVEL_HOST, out, "A3", status );
+	}
+
+	append_people ( in, "EDITOR",      LEVEL_SERIES, out, "A3", status );
+	append_easyall( in, "EDITOR:CORP", LEVEL_SERIES, out, "A3", status );
+	append_easyall( in, "EDITOR:ASIS", LEVEL_SERIES, out, "A3", status );
 }
 
-static void
-output( FILE *fp, fields *out )
+static int
+risout_assemble( fields *in, fields *out, param *pm, unsigned long refnum )
 {
-	char *tag, *value;
+	int type, status = BIBL_OK;
+
+	type = get_type( in, pm );
+
+	append_type      ( type, pm, out, &status );
+	append_allpeople ( in, type, out, &status );
+	append_date      ( in, out, &status );
+	append_alltitles ( in, type, out, &status );
+	append_pages     ( in, out, &status );
+	append_easy      ( in, "VOLUME",             LEVEL_ANY, out, "VL", &status );
+	append_easy      ( in, "ISSUE",              LEVEL_ANY, out, "IS", &status );
+	append_easy      ( in, "NUMBER",             LEVEL_ANY, out, "IS", &status );
+	append_easy      ( in, "EDITION",            LEVEL_ANY, out, "ET", &status );
+	append_easy      ( in, "NUMVOLUMES",         LEVEL_ANY, out, "NV", &status );
+	append_easycombo ( in, "ADDRESS:AUTHOR",     LEVEL_ANY, out, "AD", "; ", &status );
+	append_easy      ( in, "PUBLISHER",          LEVEL_ANY, out, "PB", &status );
+	append_easy      ( in, "DEGREEGRANTOR",      LEVEL_ANY, out, "PB", &status );
+	append_easy      ( in, "DEGREEGRANTOR:ASIS", LEVEL_ANY, out, "PB", &status );
+	append_easy      ( in, "DEGREEGRANTOR:CORP", LEVEL_ANY, out, "PB", &status );
+	append_easycombo ( in, "ADDRESS",            LEVEL_ANY, out, "CY", "; ", &status );
+	append_easyall   ( in, "KEYWORD",            LEVEL_ANY, out, "KW", &status );
+	append_easy      ( in, "ABSTRACT",           LEVEL_ANY, out, "AB", &status );
+	append_easy      ( in, "CALLNUMBER",         LEVEL_ANY, out, "CN", &status );
+	append_easy      ( in, "ISSN",               LEVEL_ANY, out, "SN", &status );
+	append_easy      ( in, "ISBN",               LEVEL_ANY, out, "SN", &status );
+	append_file      ( in, "FILEATTACH",         LEVEL_ANY, out, "L1", &status );
+	append_file      ( in, "FIGATTACH",          LEVEL_ANY, out, "L4", &status );
+	append_easy      ( in, "CAPTION",            LEVEL_ANY, out, "CA", &status );
+	append_urls      ( in, out, &status );
+	append_easyall   ( in, "DOI",                LEVEL_ANY, out, "DO", &status );
+	append_easy      ( in, "LANGUAGE",           LEVEL_ANY, out, "LA", &status );
+	append_easy      ( in, "NOTES",              LEVEL_ANY, out, "N1", &status );
+	append_easy      ( in, "REFNUM",             LEVEL_ANY, out, "ID", &status );
+	append_thesishint( type, out, &status );
+
+	return status;
+}
+
+/*****************************************************
+ PUBLIC: int risout_write()
+*****************************************************/
+
+static int
+risout_write( fields *out, FILE *fp, param *p, unsigned long refnum )
+{
+	const char *tag, *value;
 	int i;
 
 	for ( i=0; i<out->n; ++i ) {
-		tag   = fields_tag( out, i, FIELDS_CHRP );
+		tag   = fields_tag  ( out, i, FIELDS_CHRP );
 		value = fields_value( out, i, FIELDS_CHRP );
 		fprintf( fp, "%s  - %s\n", tag, value );
 	}
 
 	fprintf( fp, "ER  - \n" );
 	fflush( fp );
-}
-
-static int
-append_data( fields *in, param *p, fields *out )
-{
-	int type, status = BIBL_OK;
-
-	type = get_type( in, p );
-
-	append_type      ( type, p, out, &status );
-	append_allpeople ( in, type, out, &status );
-	append_date      ( in, out, &status );
-	append_alltitles ( in, type, out, &status );
-	append_pages     ( in, out, &status );
-	append_easy      ( in, "VOLUME",             "VL", LEVEL_ANY, out, &status );
-	append_easy      ( in, "ISSUE",              "IS", LEVEL_ANY, out, &status );
-	append_easy      ( in, "NUMBER",             "IS", LEVEL_ANY, out, &status );
-	append_easy      ( in, "EDITION",            "ET", LEVEL_ANY, out, &status );
-	append_easy      ( in, "NUMVOLUMES",         "NV", LEVEL_ANY, out, &status );
-	append_easy      ( in, "AUTHORADDRESS",      "AD", LEVEL_ANY, out, &status );
-	append_easy      ( in, "PUBLISHER",          "PB", LEVEL_ANY, out, &status );
-	append_easy      ( in, "DEGREEGRANTOR",      "PB", LEVEL_ANY, out, &status );
-	append_easy      ( in, "DEGREEGRANTOR:ASIS", "PB", LEVEL_ANY, out, &status );
-	append_easy      ( in, "DEGREEGRANTOR:CORP", "PB", LEVEL_ANY, out, &status );
-	append_easy      ( in, "ADDRESS",            "CY", LEVEL_ANY, out, &status );
-	append_keywords  ( in, out, &status );
-	append_easy      ( in, "ABSTRACT",           "AB", LEVEL_ANY, out, &status );
-	append_easy      ( in, "CALLNUMBER",         "CN", LEVEL_ANY, out, &status );
-	append_easy      ( in, "ISSN",               "SN", LEVEL_ANY, out, &status );
-	append_easy      ( in, "ISBN",               "SN", LEVEL_ANY, out, &status );
-	append_file      ( in, "FILEATTACH",         "L1", LEVEL_ANY, out, &status );
-	append_file      ( in, "FIGATTACH",          "L4", LEVEL_ANY, out, &status );
-	append_easy      ( in, "CAPTION",            "CA", LEVEL_ANY, out, &status );
-	append_urls      ( in, out, &status );
-	append_easyall   ( in, "DOI",                "DO", LEVEL_ANY, out, &status );
-	append_easy      ( in, "LANGUAGE",           "LA", LEVEL_ANY, out, &status );
-	append_easy      ( in, "NOTES",              "N1", LEVEL_ANY, out, &status );
-	append_easy      ( in, "REFNUM",             "ID", LEVEL_ANY, out, &status );
-	append_thesishint( type, out, &status );
-
-	return status;
-}
-
-static int
-risout_write( fields *in, FILE *fp, param *p, unsigned long refnum )
-{
-	int status;
-	fields out;
-
-	fields_init( &out );
-
-	status = append_data( in, p, &out );
-	if ( status==BIBL_OK ) output( fp, &out );
-
-	fields_free( &out );
-
-	return status;
-}
-
-static void
-risout_writeheader( FILE *outptr, param *p )
-{
-	if ( p->utf8bom ) utf8_writebom( outptr );
+	return BIBL_OK;
 }
