@@ -1,9 +1,9 @@
 /*
  * str.c
  *
- * Version: 2017-07-03
+ * Version: 2018-09-21
  *
- * Copyright (c) Chris Putnam 1999-2017
+ * Copyright (c) Chris Putnam 1999-2021
  *
  * Source code released under the GPL version 2
  *
@@ -28,6 +28,41 @@
 
 #define str_initlen (64)
 
+#ifndef STR_SMALL
+
+#define str_clear_status( s ) s->status = STR_OK;
+#define handle_memerr( s, f ) s->status = STR_MEMERR;
+#define return_if_memerr( s ) \
+{ \
+	if ( s->status != STR_OK ) return; \
+}
+#define return_zero_if_memerr( s ) \
+{ \
+	if ( s->status != STR_OK ) return 0; \
+}
+#define return_after_delim_if_memerr( s, p, delim, finalstep ) \
+{ \
+	if ( s->status != STR_OK ) { \
+		while ( p && *p && !strchr( delim, *p ) ) p++; \
+		if ( p && *p && finalstep ) p++; \
+		return p; \
+	} \
+}
+
+#else
+
+#define str_clear_status( s ) {}
+#define handle_memerr( s, f ) \
+{ \
+	fprintf( stderr,"Error.  Cannot allocate memory in %s.\n", f ); \
+	exit( EXIT_FAILURE ); \
+}
+#define return_if_memerr( s ) {}
+#define return_zero_if_memerr( s ) {}
+#define return_after_delim_if_memerr( s, p, delim, finalstep ) {}
+
+#endif
+
 
 /* Clear memory in resize/free if STR_PARANOIA defined */
 
@@ -36,16 +71,18 @@
 static void 
 str_realloc( str *s, unsigned long minsize )
 {
-	char *newptr;
 	unsigned long size;
+	char *newptr;
+
 	assert( s );
+	return_if_memerr( s );
+
 	size = 2 * s->dim;
 	if (size < minsize) size = minsize;
+
 	newptr = (char *) realloc( s->data, sizeof( *(s->data) )*size );
-	if ( !newptr ) {
-		fprintf(stderr,"Error.  Cannot reallocate memory (%ld bytes) in str_realloc.\n", sizeof(*(s->data))*size);
-		exit( EXIT_FAILURE );
-	}
+	if ( !newptr ) handle_memerr( s, __FUNCTION__ );
+
 	s->data = newptr;
 	s->dim = size;
 }
@@ -58,17 +95,18 @@ str_realloc( str *s, unsigned long minsize )
 static void 
 str_realloc( str *s, unsigned long minsize )
 {
-	char *newptr;
 	unsigned long size;
+	char *newptr;
+
 	assert( s );
+	return_if_memerr( s );
+
 	size = 2 * s->dim;
 	if ( size < minsize ) size = minsize;
+
 	newptr = (char *) malloc( sizeof( *(s->data) ) * size );
-	if ( !newptr ) {
-		fprintf( stderr, "Error.  Cannot reallocate memory (%d bytes)"
-			" in str_realloc.\n", sizeof(*(s->data))*size );
-		exit( EXIT_FAILURE );
-	}
+	if ( !newptr ) handle_memerr( s, __FUNCTION__ );
+
 	if ( s->data ) {
 		str_nullify( s );
 		free( s->data );
@@ -92,6 +130,7 @@ str_init( str *s )
 	s->dim = 0;
 	s->len = 0;
 	s->data = NULL;
+	str_clear_status( s );
 }
 
 void
@@ -140,16 +179,14 @@ strs_init( str *s, ... )
 	va_end( ap );
 }
 
-/*
- * This is currently a stub. Later it will
- * report whether or not a str function
- * could not be performed due to a memory
- * error.
- */
 int
 str_memerr( str *s )
 {
+#ifndef STR_SMALL
+	return s->status == STR_MEMERR;
+#else
 	return 0;
+#endif
 }
 
 void
@@ -157,6 +194,7 @@ str_mergestrs( str *s, ... )
 {
 	va_list ap;
 	const char *cp;
+	str_clear_status( s );
 	str_empty( s );
 	va_start( ap, s );
 	do {
@@ -172,14 +210,15 @@ str_initalloc( str *s, unsigned long minsize )
 	unsigned long size = str_initlen;
 	assert( s );
 	if ( minsize > str_initlen ) size = minsize;
-	s->data = (char *) malloc (sizeof( *(s->data) ) * size);
+	s->data = (char *) malloc( sizeof( *(s->data) ) * size );
 	if ( !s->data ) {
-		fprintf(stderr,"Error.  Cannot allocate memory in str_initalloc.\n");
+		fprintf(stderr,"Error.  Cannot allocate memory in str_initalloc, requested %lu characters.\n", size );
 		exit( EXIT_FAILURE );
 	}
 	s->data[0]='\0';
 	s->dim=size;
 	s->len=0;
+	str_clear_status( s );
 }
 
 str *
@@ -230,6 +269,7 @@ void
 str_empty( str *s )
 {
 	assert( s );
+	str_clear_status( s );
 	if ( s->data ) {
 		str_nullify( s );
 		s->data[0] = '\0';
@@ -255,11 +295,16 @@ void
 str_addchar( str *s, char newchar )
 {
 	assert( s );
+
+	return_if_memerr( s );
+
 	if ( newchar=='\0' ) return; /* appending '\0' is a null operation */
+
 	if ( !s->data || s->dim==0 ) 
 		str_initalloc( s, str_initlen );
 	if ( s->len + 2 > s->dim ) 
 		str_realloc( s, s->len*2 );
+
 	s->data[s->len++] = newchar;
 	s->data[s->len] = '\0';
 }
@@ -305,9 +350,14 @@ void
 str_prepend( str *s, const char *addstr )
 {
 	unsigned long lenaddstr, i;
+
 	assert( s && addstr );
+
+	return_if_memerr( s );
+
 	lenaddstr = strlen( addstr );
-	if ( lenaddstr==0 ) return;
+	if ( lenaddstr==0 ) return; /* appending an empty string is a null op */
+
 	if ( !s->data || !s->dim )
 		str_initalloc( s, lenaddstr+1 );
 	else {
@@ -334,6 +384,7 @@ str_strcat_ensurespace( str *s, unsigned long n )
 static inline void 
 str_strcat_internal( str *s, const char *addstr, unsigned long n )
 {
+	return_if_memerr( s );
 	str_strcat_ensurespace( s, n );
 	strncat( &(s->data[s->len]), addstr, n );
 	s->len += n;
@@ -366,6 +417,8 @@ str_segcat( str *s, char *startat, char *endat )
 	assert( s && startat && endat );
 	assert( (size_t) startat < (size_t) endat );
 
+	return_if_memerr( s );
+
 	if ( startat==endat ) return;
 
 	n = 0;
@@ -382,8 +435,12 @@ void
 str_indxcat( str *s, char *p, unsigned long start, unsigned long stop )
 {
 	unsigned long i;
+
 	assert( s && p );
 	assert( start <= stop );
+
+	return_if_memerr( s );
+
 	for ( i=start; i<stop; ++i )
 		str_addchar( s, p[i] );
 }
@@ -393,10 +450,11 @@ str_indxcat( str *s, char *p, unsigned long start, unsigned long stop )
  *     finalstep = set to non-zero to position return value past the
  *                 terminating character
  */
-char *
-str_cpytodelim( str *s, char *p, const char *delim, unsigned char finalstep )
+const char *
+str_cpytodelim( str *s, const char *p, const char *delim, unsigned char finalstep )
 {
 	assert( s );
+
 	str_empty( s );
 	return str_cattodelim( s, p, delim, finalstep );
 }
@@ -406,10 +464,13 @@ str_cpytodelim( str *s, char *p, const char *delim, unsigned char finalstep )
  *     finalstep = set to non-zero to position return value past the
  *                 terminating character
  */
-char *
-str_cattodelim( str *s, char *p, const char *delim, unsigned char finalstep )
+const char *
+str_cattodelim( str *s, const char *p, const char *delim, unsigned char finalstep )
 {
 	assert( s );
+
+	return_after_delim_if_memerr( s, p, delim, finalstep );
+
 	while ( p && *p && !strchr( delim, *p ) ) {
 		str_addchar( s, *p );
 		p++;
@@ -431,6 +492,8 @@ str_strcpy_ensurespace( str *s, unsigned long n )
 static inline void
 str_strcpy_internal( str *s, const char *p, unsigned long n )
 {
+	return_if_memerr( s );
+
 	str_strcpy_ensurespace( s, n );
 	strncpy( s->data, p, n );
 	s->data[n] = '\0';
@@ -469,6 +532,8 @@ str_segcpy( str *s, char *startat, char *endat )
 	assert( s && startat && endat );
 	assert( ((size_t) startat) <= ((size_t) endat) );
 
+	return_if_memerr( s );
+
 	if ( startat==endat ) {
 		str_empty( s );
 		return;
@@ -493,8 +558,12 @@ void
 str_indxcpy( str *s, char *p, unsigned long start, unsigned long stop )
 {
 	unsigned long i;
+
 	assert( s && p );
 	assert( start <= stop );
+
+	return_if_memerr( s );
+
 	if ( start == stop ) {
 		str_empty( s );
 		return;
@@ -529,7 +598,11 @@ str_segdel( str *s, char *p, char *q )
 {
 	str tmp1, tmp2;
 	char *r;
+
 	assert( s );
+
+	return_if_memerr( s );
+
 	r = &(s->data[s->len]);
 	str_init( &tmp1 );
 	str_init( &tmp2 );
@@ -561,7 +634,10 @@ str_findreplace( str *s, const char *find, const char *replace )
 	int n = 0;
 
 	assert( s && find );
-	if ( !s->data || !s->dim ) return n;
+
+	return_zero_if_memerr( s );
+
+	if ( !s->data || !s->dim ) return 0;
 	if ( !replace ) replace = empty;
 
 	find_len = strlen( find );
@@ -1016,6 +1092,7 @@ str_fill( str *s, unsigned long n, char fillchar )
 {
 	unsigned long i;
 	assert( s );
+	str_clear_status( s );
 	if ( !s->data || s->dim==0 )
 		str_initalloc( s, n+1 );
 	if ( n + 1 > s->dim )
@@ -1039,3 +1116,10 @@ str_is_empty( str *s )
 	if ( !s || s->len==0 ) return 1;
 	return 0;
 }
+
+unsigned long
+str_strlen( str *s )
+{
+	if ( !s ) return 0;
+	else return s->len;
+}
