@@ -1,7 +1,7 @@
 /*
  * unicode.h
  *
- * Copyright (c) Chris Putnam 2004-2017
+ * Copyright (c) Chris Putnam 2004-2021
  *
  * Program and source code released under the GPL version 2
  */
@@ -16,7 +16,7 @@
 #define UNICODE_NUMBER    (8) /* Numeric character */
 #define UNICODE_MIXEDCASE ( UNICODE_UPPER | UNICODE_LOWER )
 
-extern unsigned short unicode_utf8_classify( char *p );
-extern unsigned short unicode_utf8_classify_str( str *s );
+unsigned short unicode_utf8_classify( char *p );
+unsigned short unicode_utf8_classify_str( str *s );
 
 #endif
