@@ -3,7 +3,7 @@
  * 
  * (Word 2007 format)
  *
- * Copyright (c) Chris Putnam 2007-2017
+ * Copyright (c) Chris Putnam 2007-2021
  *
  * Source code released under the GPL version 2
  *
@@ -13,37 +13,57 @@
 #include <string.h>
 #include "str.h"
 #include "fields.h"
+#include "month.h"
 #include "utf8.h"
 #include "bibformats.h"
 
+/*****************************************************
+ PUBLIC: int wordout_initparams()
+*****************************************************/
+
 static void wordout_writeheader( FILE *outptr, param *p );
 static void wordout_writefooter( FILE *outptr );
 static int  wordout_write( fields *info, FILE *outptr, param *p, unsigned long numrefs );
 
-void
-wordout_initparams( param *p, const char *progname )
+int
+wordout_initparams( param *pm, const char *progname )
 {
-	p->writeformat      = BIBL_WORD2007OUT;
-	p->format_opts      = 0;
-	p->charsetout       = BIBL_CHARSET_UNICODE;
-	p->charsetout_src   = BIBL_SRC_DEFAULT;
-	p->latexout         = 0;
-	p->utf8out          = BIBL_CHARSET_UTF8_DEFAULT;
-	p->utf8bom          = BIBL_CHARSET_BOM_DEFAULT;
-	if ( !p->utf8out )
-		p->xmlout   = BIBL_XMLOUT_ENTITIES;
+	pm->writeformat      = BIBL_WORD2007OUT;
+	pm->format_opts      = 0;
+	pm->charsetout       = BIBL_CHARSET_UNICODE;
+	pm->charsetout_src   = BIBL_SRC_DEFAULT;
+	pm->latexout         = 0;
+	pm->utf8out          = BIBL_CHARSET_UTF8_DEFAULT;
+	pm->utf8bom          = BIBL_CHARSET_BOM_DEFAULT;
+	if ( !pm->utf8out )
+		pm->xmlout   = BIBL_XMLOUT_ENTITIES;
 	else
-		p->xmlout   = BIBL_XMLOUT_TRUE;
-	p->nosplittitle     = 0;
-	p->verbose          = 0;
-	p->addcount         = 0;
-	p->singlerefperfile = 0;
+		pm->xmlout   = BIBL_XMLOUT_TRUE;
+	pm->nosplittitle     = 0;
+	pm->verbose          = 0;
+	pm->addcount         = 0;
+	pm->singlerefperfile = 0;
 
-	p->headerf = wordout_writeheader;
-	p->footerf = wordout_writefooter;
-	p->writef  = wordout_write;
+	pm->headerf   = wordout_writeheader;
+	pm->footerf   = wordout_writefooter;
+	pm->assemblef = NULL;
+	pm->writef    = wordout_write;
+
+	if ( !pm->progname ) {
+		if ( !progname ) pm->progname = NULL;
+		else {
+			pm->progname = strdup( progname );
+			if ( !pm->progname ) return BIBL_ERR_MEMERR;
+		}
+	}
+
+	return BIBL_OK;
 }
 
+/*****************************************************
+ PUBLIC: int wordout_write()
+*****************************************************/
+
 typedef struct convert {
 	char *oldtag;
 	char *newtag;
@@ -103,39 +123,47 @@ enum {
 	TYPE_PHDTHESIS,
 };
 
-/*
- * fixed output
- */
 static void
-output_fixed( FILE *outptr, char *tag, char *value, int level )
+output_level( FILE *outptr, int level )
 {
 	int i;
-	for ( i=0; i<level; ++i ) fprintf( outptr, " " );
+	for ( i=0; i<level; ++i )
+		fprintf( outptr, " " );
+}
+
+/* fixed output
+ *
+ * <TAG>value</TAG>
+ */
+static void
+output_fixed( FILE *outptr, const char *tag, const char *value, int level )
+{
+	output_level( outptr, level );
 	fprintf( outptr, "<%s>%s</%s>\n", tag, value, tag );
 }
 
 /* detail output
  *
+ * <TAG>value</TAG>
  */
 static void
-output_item( fields *info, FILE *outptr, char *tag, char *prefix, int item, int level )
+output_item( fields *info, FILE *outptr, const char *tag, const char *prefix, int item, int level )
 {
-	int i;
-	if ( item==-1 ) return;
-	for ( i=0; i<level; ++i ) fprintf( outptr, " " );
-	fprintf( outptr, "<%s>%s%s</%s>\n",
-		tag,
-		prefix,
-		(char*) fields_value( info, item, FIELDS_CHRP ),
-		tag
-	);
+	if ( item!=-1 ) {
+		output_level( outptr, level );
+		fprintf( outptr, "<%s>%s%s</%s>\n",
+			tag,
+			prefix,
+			(char*) fields_value( info, item, FIELDS_CHRP ),
+			tag
+		);
+	}
 }
 
 static void
-output_itemv( FILE *outptr, char *tag, char *item, int level )
+output_itemv( FILE *outptr, const char *tag, const char *item, int level )
 {
-	int i;
-	for ( i=0; i<level; ++i ) fprintf( outptr, " " );
+	output_level( outptr, level );
 	fprintf( outptr, "<%s>%s</%s>\n", tag, item, tag );
 }
 
@@ -145,20 +173,29 @@ output_itemv( FILE *outptr, char *tag, char *item, int level )
  *
  */
 static void
-output_range( FILE *outptr, char *tag, char *start, char *end, int level )
+output_nodash( FILE *outptr, const char *p )
 {
-	int i;
-	if ( start==NULL && end==NULL ) return;
-	if ( start==NULL )
-		output_itemv( outptr, tag, end, 0 );
-	else if ( end==NULL )
-		output_itemv( outptr, tag, start, 0 );
-	else {
-		for ( i=0; i<level; ++i )
-			fprintf( outptr, " " );
-		fprintf( outptr, "<%s>%s-%s</%s>\n", tag, start, end, tag );
+	while ( *p ) {
+		/* -30 is the first character of a UTF8 em-dash and en-dash */
+		if ( *p==-30 && ( utf8_is_emdash( p ) || utf8_is_endash( p ) ) ) {
+			fprintf( outptr, "-" );
+			p+=3;
+		} else {
+			fprintf( outptr, "%c", *p );
+			p+=1;
+		}
 	}
 }
+static void
+output_range( FILE *outptr, const char *tag, const char *start, const char *end, int level )
+{
+	if ( !start && !end ) return;
+	fprintf( outptr, "<%s>", tag );
+	if ( start ) output_nodash( outptr, start );
+	if ( start && end ) fprintf( outptr, "-" );
+	if ( end ) output_nodash( outptr, end );
+	fprintf( outptr, "</%s>\n", tag );
+}
 
 static void
 output_list( fields *info, FILE *outptr, convert *c, int nc )
@@ -166,37 +203,38 @@ output_list( fields *info, FILE *outptr, convert *c, int nc )
         int i, n;
         for ( i=0; i<nc; ++i ) {
                 n = fields_find( info, c[i].oldtag, c[i].code );
-                if ( n!=-1 ) output_item( info, outptr, c[i].newtag, c[i].prefix, n, 0 );
+                if ( n!=FIELDS_NOTFOUND ) output_item( info, outptr, c[i].newtag, c[i].prefix, n, 0 );
         }
 
 }
 
 typedef struct outtype {
 	int value;
-	char *out;
+	const char *out;
 } outtype;
 
 static
 outtype genres[] = {
-	{ TYPE_PATENT, "patent" },
-	{ TYPE_REPORT, "report" },
-	{ TYPE_CASE,   "legal case and case notes" },
-	{ TYPE_ART,    "art original" },
-	{ TYPE_ART,    "art reproduction" },
-	{ TYPE_ART,    "comic strip" },
-	{ TYPE_ART,    "diorama" },
-	{ TYPE_ART,    "graphic" },
-	{ TYPE_ART,    "model" },
-	{ TYPE_ART,    "picture" },
+	{ TYPE_PATENT,           "patent" },
+	{ TYPE_REPORT,           "report" },
+	{ TYPE_REPORT,           "technical report" },
+	{ TYPE_CASE,             "legal case and case notes" },
+	{ TYPE_ART,              "art original" },
+	{ TYPE_ART,              "art reproduction" },
+	{ TYPE_ART,              "comic strip" },
+	{ TYPE_ART,              "diorama" },
+	{ TYPE_ART,              "graphic" },
+	{ TYPE_ART,              "model" },
+	{ TYPE_ART,              "picture" },
 	{ TYPE_ELECTRONICSOURCE, "electronic" },
-	{ TYPE_FILM,   "videorecording" },
-	{ TYPE_FILM,   "motion picture" },
-	{ TYPE_SOUNDRECORDING, "sound" },
-	{ TYPE_PERFORMANCE, "rehersal" },
-	{ TYPE_INTERNETSITE, "web site" },
-	{ TYPE_INTERVIEW, "interview" },
-	{ TYPE_INTERVIEW, "communication" },
-	{ TYPE_MISC, "misc" },
+	{ TYPE_FILM,             "videorecording" },
+	{ TYPE_FILM,             "motion picture" },
+	{ TYPE_SOUNDRECORDING,   "sound" },
+	{ TYPE_PERFORMANCE,      "rehersal" },
+	{ TYPE_INTERNETSITE,     "web site" },
+	{ TYPE_INTERVIEW,        "interview" },
+	{ TYPE_INTERVIEW,        "communication" },
+	{ TYPE_MISC,             "misc" },
 };
 int ngenres = sizeof( genres ) / sizeof( genres[0] );
 
@@ -204,17 +242,17 @@ static int
 get_type_from_genre( fields *info )
 {
 	int type = TYPE_UNKNOWN, i, j, level;
-	char *genre, *tag;
+	const char *genre, *tag;
 	for ( i=0; i<info->n; ++i ) {
-		tag = (char *) fields_tag( info, i, FIELDS_CHRP );
-		if ( strcasecmp( tag, "GENRE" ) && strcasecmp( tag, "NGENRE" ) ) continue;
-		genre = (char *) fields_value( info, i, FIELDS_CHRP );
+		tag = (const char *) fields_tag( info, i, FIELDS_CHRP );
+		if ( strcasecmp( tag, "GENRE:MARC" ) && strcasecmp( tag, "GENRE:BIBUTILS" ) && strcasecmp( tag, "GENRE:UNKNOWN" ) ) continue;
+		genre = (const char *) fields_value( info, i, FIELDS_CHRP );
 		for ( j=0; j<ngenres; ++j ) {
 			if ( !strcasecmp( genres[j].out, genre ) )
 				type = genres[j].value;
 		}
 		if ( type==TYPE_UNKNOWN ) {
-			level = info->level[i];
+			level = fields_level( info, i );
 			if ( !strcasecmp( genre, "academic journal" ) ) {
 				type = TYPE_JOURNALARTICLE;
 			}
@@ -224,7 +262,7 @@ get_type_from_genre( fields *info )
 			}
 			else if ( !strcasecmp( genre, "book" ) ||
 				!strcasecmp( genre, "collection" ) ) {
-				if ( info->level[i]==0 ) type = TYPE_BOOK;
+				if ( fields_level( info, i ) == 0 ) type = TYPE_BOOK;
 				else type = TYPE_BOOKSECTION;
 			}
 			else if ( !strcasecmp( genre, "conference publication" ) ) {
@@ -252,7 +290,7 @@ get_type_from_resource( fields *info )
 	char *tag, *resource;
 	for ( i=0; i<info->n; ++i ) {
 		tag = (char *) fields_tag( info, i, FIELDS_CHRP );
-		if ( strcasecmp( tag, "GENRE" ) && strcasecmp( tag, "NGENRE" ) ) continue;
+		if ( strcasecmp( tag, "RESOURCE" ) ) continue;
 		resource = (char *) fields_value( info, i, FIELDS_CHRP );
 		if ( !strcasecmp( resource, "moving image" ) )
 			type = TYPE_FILM;
@@ -271,7 +309,7 @@ get_type( fields *info )
 }
 
 static void
-output_titlebits( char *mainttl, char *subttl, FILE *outptr )
+output_titlebits( const char *mainttl, const char *subttl, FILE *outptr )
 {
 	if ( mainttl ) fprintf( outptr, "%s", mainttl );
 	if ( subttl ) {
@@ -285,7 +323,7 @@ output_titlebits( char *mainttl, char *subttl, FILE *outptr )
 }
 
 static void
-output_titleinfo( char *mainttl, char *subttl, FILE *outptr, char *tag, int level )
+output_titleinfo( const char *mainttl, const char *subttl, FILE *outptr, const char *tag, int level )
 {
 	if ( mainttl || subttl ) {
 		fprintf( outptr, "<%s>", tag );
@@ -295,12 +333,12 @@ output_titleinfo( char *mainttl, char *subttl, FILE *outptr, char *tag, int leve
 }
 
 static void
-output_generaltitle( fields *info, FILE *outptr, char *tag, int level )
+output_generaltitle( fields *info, FILE *outptr, const char *tag, int level )
 {
-	char *ttl       = fields_findv( info, level, FIELDS_CHRP, "TITLE" );
-	char *subttl    = fields_findv( info, level, FIELDS_CHRP, "SUBTITLE" );
-	char *shrttl    = fields_findv( info, level, FIELDS_CHRP, "SHORTTITLE" );
-	char *shrsubttl = fields_findv( info, level, FIELDS_CHRP, "SHORTSUBTITLE" );
+	const char *ttl       = fields_findv( info, level, FIELDS_CHRP, "TITLE" );
+	const char *subttl    = fields_findv( info, level, FIELDS_CHRP, "SUBTITLE" );
+	const char *shrttl    = fields_findv( info, level, FIELDS_CHRP, "SHORTTITLE" );
+	const char *shrsubttl = fields_findv( info, level, FIELDS_CHRP, "SHORTSUBTITLE" );
 
 	if ( ttl ) {
 		output_titleinfo( ttl, subttl, outptr, tag, level );
@@ -313,10 +351,10 @@ output_generaltitle( fields *info, FILE *outptr, char *tag, int level )
 static void
 output_maintitle( fields *info, FILE *outptr, int level )
 {
-	char *ttl       = fields_findv( info, level, FIELDS_CHRP, "TITLE" );
-	char *subttl    = fields_findv( info, level, FIELDS_CHRP, "SUBTITLE" );
-	char *shrttl    = fields_findv( info, level, FIELDS_CHRP, "SHORTTITLE" );
-	char *shrsubttl = fields_findv( info, level, FIELDS_CHRP, "SHORTSUBTITLE" );
+	const char *ttl       = fields_findv( info, level, FIELDS_CHRP, "TITLE" );
+	const char *subttl    = fields_findv( info, level, FIELDS_CHRP, "SUBTITLE" );
+	const char *shrttl    = fields_findv( info, level, FIELDS_CHRP, "SHORTTITLE" );
+	const char *shrsubttl = fields_findv( info, level, FIELDS_CHRP, "SHORTSUBTITLE" );
 
 	if ( ttl ) {
 		output_titleinfo( ttl, subttl, outptr, "b:Title", level );
@@ -336,7 +374,7 @@ output_maintitle( fields *info, FILE *outptr, int level )
 }
 
 static void
-output_name_nomangle( FILE *outptr, char *p )
+output_name_nomangle( FILE *outptr, const char *p )
 {
 	fprintf( outptr, "<b:Person>" );
 	fprintf( outptr, "<b:Last>%s</b:Last>", p );
@@ -344,7 +382,7 @@ output_name_nomangle( FILE *outptr, char *p )
 }
 
 static void
-output_name( FILE *outptr, char *p )
+output_name( FILE *outptr, const char *p )
 {
 	str family, part;
 	int n=0, npart=0;
@@ -397,8 +435,7 @@ extract_name_and_info( str *outtag, str *intag )
 }
 
 static void
-output_name_type( fields *info, FILE *outptr, int level, 
-			char *map[], int nmap, char *tag )
+output_name_type( fields *info, FILE *outptr, int level, char *map[], int nmap, const char *tag )
 {
 	str ntag;
 	int i, j, n=0, code, nfields;
@@ -406,7 +443,7 @@ output_name_type( fields *info, FILE *outptr, int level,
 	nfields = fields_num( info );
 	for ( j=0; j<nmap; ++j ) {
 		for ( i=0; i<nfields; ++i ) {
-			code = extract_name_and_info( &ntag, &(info->tag[i]) );
+			code = extract_name_and_info( &ntag, fields_tag( info, i, FIELDS_STRP ) );
 			if ( strcasecmp( str_cstr( &ntag ), map[j] ) ) continue;
 			if ( n==0 )
 				fprintf( outptr, "<%s><b:NameList>\n", tag );
@@ -427,7 +464,7 @@ output_names( fields *info, FILE *outptr, int level, int type )
 {
 	char *authors[] = { "AUTHOR", "WRITER", "ASSIGNEE", "ARTIST",
 		"CARTOGRAPHER", "INVENTOR", "ORGANIZER", "DIRECTOR",
-		"PERFORMER", "REPORTER", "TRANSLATOR", "RECIPIENT",
+		"PERFORMER", "REPORTER", "TRANSLATOR", "ADDRESSEE",
 		"2ND_AUTHOR", "3RD_AUTHOR", "SUB_AUTHOR", "COMMITTEE",
 		"COURT", "LEGISLATIVEBODY" };
 	int nauthors = sizeof( authors ) / sizeof( authors[0] );
@@ -449,23 +486,28 @@ output_names( fields *info, FILE *outptr, int level, int type )
 static void
 output_date( fields *info, FILE *outptr, int level )
 {
-	char *year  = fields_findv_firstof( info, level, FIELDS_CHRP,
+	const char *use;
+
+	const char *year  = fields_findv_firstof( info, level, FIELDS_CHRP,
 			"PARTDATE:YEAR", "DATE:YEAR", NULL );
-	char *month = fields_findv_firstof( info, level, FIELDS_CHRP,
+	const char *month = fields_findv_firstof( info, level, FIELDS_CHRP,
 			"PARTDATE:MONTH", "DATE:MONTH", NULL );
-	char *day   = fields_findv_firstof( info, level, FIELDS_CHRP,
+	const char *day   = fields_findv_firstof( info, level, FIELDS_CHRP,
 			"PARTDATE:DAY", "DATE:DAY", NULL );
 	if ( year )  output_itemv( outptr, "b:Year", year, 0 );
-	if ( month ) output_itemv( outptr, "b:Month", month, 0 );
+	if ( month ) {
+		(void) number_to_full_month( month, &use );
+		output_itemv( outptr, "b:Month", use, 0 );
+	}
 	if ( day )   output_itemv( outptr, "b:Day", day, 0 );
 }
 
 static void
 output_pages( fields *info, FILE *outptr, int level )
 {
-	char *sn = fields_findv( info, LEVEL_ANY, FIELDS_CHRP, "PAGES:START" );
-	char *en = fields_findv( info, LEVEL_ANY, FIELDS_CHRP, "PAGES:STOP" );
-	char *ar = fields_findv( info, LEVEL_ANY, FIELDS_CHRP, "ARTICLENUMBER" );
+	const char *sn = fields_findv( info, LEVEL_ANY, FIELDS_CHRP, "PAGES:START" );
+	const char *en = fields_findv( info, LEVEL_ANY, FIELDS_CHRP, "PAGES:STOP" );
+	const char *ar = fields_findv( info, LEVEL_ANY, FIELDS_CHRP, "ARTICLENUMBER" );
 	if ( sn || en )
 		output_range( outptr, "b:Pages", sn, en, level );
 	else if ( ar )
@@ -566,9 +608,9 @@ output_type( fields *info, FILE *outptr, int type )
 static void
 output_comments( fields *info, FILE *outptr, int level )
 {
+	const char *abs;
 	vplist_index i;
 	vplist notes;
-	char *abs;
 
 	vplist_init( &notes );
 
@@ -587,7 +629,7 @@ output_comments( fields *info, FILE *outptr, int level )
 static void
 output_bibkey( fields *info, FILE *outptr )
 {
-	char *bibkey = fields_findv_firstof( info, LEVEL_ANY, FIELDS_CHRP,
+	const char *bibkey = fields_findv_firstof( info, LEVEL_ANY, FIELDS_CHRP,
 			"REFNUM", "BIBKEY", NULL );
 	if ( bibkey ) output_itemv( outptr, "b:Tag", bibkey, 0 );
 }
@@ -646,6 +688,10 @@ wordout_write( fields *info, FILE *outptr, param *p, unsigned long numrefs )
 	return BIBL_OK;
 }
 
+/*****************************************************
+ PUBLIC: void wordout_writeheader()
+*****************************************************/
+
 static void
 wordout_writeheader( FILE *outptr, param *p )
 {
@@ -656,6 +702,10 @@ wordout_writeheader( FILE *outptr, param *p )
 		" xmlns=\"http://schemas.openxmlformats.org/officeDocument/2006/bibliography\" >\n");
 }
 
+/*****************************************************
+ PUBLIC: void wordout_writefooter()
+*****************************************************/
+
 static void
 wordout_writefooter( FILE *outptr )
 {
